/***********************************************************************************************************************
 *
 * BlueBook - use live code examples in your book
 * Copyright (C) 2011-2011 by Tidalwave s.a.s.
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebook.java.net
 * SCM: https://hg.java.net/hg/bluebook~src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebook.configuration;

import javax.annotation.Nonnull;
import java.io.File;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class ConfigurationProvider 
  {
    @Nonnull
    private final ObjectFactory objectFactory;

    public ConfigurationProvider()
      {
        this(new ObjectFactory());  
      }
    
    public ConfigurationProvider (final @Nonnull ObjectFactory objectFactory) 
      {
        this.objectFactory = objectFactory;
      }

    @Nonnull
    public Configuration loadConfiguration (final @Nonnull String fileName) 
      throws JAXBException
      {
        return loadConfiguration(new File(fileName));
      }
    
    @Nonnull
    public Configuration loadConfiguration (final @Nonnull File file) 
      throws JAXBException
      {
        final JAXBContext jc = JAXBContext.newInstance("it.tidalwave.bluebook.configuration.jaxb");
        final Unmarshaller u = jc.createUnmarshaller();        
        u.setProperty("com.sun.xml.bind.ObjectFactory", objectFactory);
        
        return ((JAXBElement<Configuration>)u.unmarshal(file)).getValue();
      }
  }
