/***********************************************************************************************************************
 *
 * BlueBook - use live code examples in your book
 * Copyright (C) 2011-2011 by Tidalwave s.a.s.
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebook.java.net
 * SCM: https://hg.java.net/hg/bluebook~src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebook.configuration;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import lombok.Getter;
import lombok.ToString;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ToString
public abstract class ConfigurationVisitor implements Cloneable
  {
    @Getter
    protected String groupId = "";
    
    @Getter
    protected String artifactId = "";
    
    @Getter
    protected String version = "";
    
    @Getter
    protected String classifier = "sources";
    
    @Getter 
    protected String type = "jar";
        
    @Getter
    protected String fileName = "";
    
    @Getter
    protected boolean skipHeader = true;
    
    @Getter 
    protected boolean skipBlockComments = true;
 
    @Getter 
    protected boolean skipPackage = false;
    
    @Getter 
    protected boolean skipImports = false;
    
    @Getter
    protected String outputDirectory = "";
    
    @Getter
    protected List<String> includedFragments = new ArrayList<String>();    
    
    @Getter
    protected List<String> excludedFragments = new ArrayList<String>();
    
    @Getter 
    protected String outputFile = "";

    @Getter 
    protected String skipRegexp = "";

    public void setArtifactId (final @Nullable String artifactId) 
      {
        this.artifactId = defaulting(artifactId, this.artifactId);
      }

    public void setClassifier (final @Nullable String classifier) 
      {
        this.classifier = defaulting(classifier, this.classifier);
      }

    public void setFileName (final @Nullable String fileName) 
      {
        this.fileName = defaulting(fileName, this.fileName);
      }

    public void setGroupId (final @Nullable String groupId) 
      {
        this.groupId = defaulting(groupId, this.groupId);
      }

    public void setType (final @Nullable String type) 
      {
        this.type = defaulting(type, this.type);
      }

    public void setVersion (final @Nullable String version) 
      {
        this.version = defaulting(version, this.version);
      }

    public void setSkipBlockComments (final @Nullable Boolean skipBlockComments) 
      {
        this.skipBlockComments = defaulting(skipBlockComments, this.skipBlockComments);
      }

    public void setSkipHeader (final @Nullable Boolean skipHeader) 
      {
        this.skipHeader = defaulting(skipHeader, this.skipHeader);
      }  

    public void setSkipImports (final @Nullable Boolean skipImports) 
      {
        this.skipImports = defaulting(skipImports, this.skipImports);
      }

    public void setSkipPackage (final @Nullable Boolean skipPackage) 
      {
        this.skipPackage = defaulting(skipPackage, this.skipPackage);
      }

    public void setSkipRegexp (final @Nullable String skipRegexp) 
      {
        this.skipRegexp = defaulting(skipRegexp, this.skipRegexp);
      }

    public void setExcludedFragments (final @Nullable List<String> excludedFragments) 
      {
        this.excludedFragments = defaulting(excludedFragments, this.excludedFragments);
      }

    public void setIncludedFragments (final @Nullable List<String> includedFragments) 
      {
        this.includedFragments = defaulting(includedFragments, this.includedFragments);
      }
    
    public void setExcludedFragments (final @Nonnull String string)
      {
        if (string != null)
          {
            setExcludedFragments(Arrays.asList(trimmed(string.split(","))));            
          }
      }

    public void setIncludedFragments (final @Nonnull String string)
      {
        if (string != null)
          {
            setIncludedFragments(Arrays.asList(trimmed(string.split(","))));            
          }
      }

    public void setOutputDirectory (final @Nullable String outputDirectory) 
      {
        this.outputDirectory = defaulting(outputDirectory, this.outputDirectory);
      }

    public void setOutputFile (final @Nullable String outputFile) 
      {
        this.outputFile = defaulting(outputFile, this.outputFile);
      }
    
    @Nonnull
    private static <T> T defaulting (final @Nullable T value, final @Nonnull T defaultValue)
      {
        return (value != null) ? value : defaultValue; 
      }
         
    @Override @Nonnull
    public ConfigurationVisitor clone()
      {
        try 
          {
            final ConfigurationVisitor clone = (ConfigurationVisitor)super.clone();
            
            clone.groupId =           this.groupId; 
            clone.artifactId =        this.artifactId;
            clone.version =           this.version;
            clone.classifier =        this.classifier;
            clone.type =              this.type;
            clone.fileName =          this.fileName;
            clone.skipHeader =        this.skipHeader;
            clone.skipBlockComments = this.skipBlockComments;
            clone.skipPackage =       this.skipPackage;
            clone.skipImports =       this.skipImports;
            clone.outputDirectory =   this.outputDirectory;
            clone.includedFragments = new ArrayList<String>(this.includedFragments);
            clone.excludedFragments = new ArrayList<String>(this.excludedFragments);
            clone.outputFile =        this.outputFile;
            
            return clone;
          } 
        catch (CloneNotSupportedException e)
          {
            throw new RuntimeException(e);
          }
      }

    public abstract void extractFile()
      throws Exception;
    
    @Nonnull
    private String[] trimmed (final @Nonnull String[] strings)
      {
        final String[] result = new String[strings.length];
        
        for (int i = 0; i < result.length; i++)
          {
            result[i] = strings[i].trim();  
          }
        
        return result;
      }
  }
